package org.xmldb.xupdate.unittests;

/**
 * (c) 2002 X-Hive Corporation B.V. (www.x-hive.com)
 * XhiveNodeMatcher
 *
 * [DESCRIPTION]
 * Compares two nodes.
 *
 */

import junit.framework.Assert;
import org.w3c.dom.*;

public class XhiveNodeMatcher {

  public XhiveNodeMatcher() {
  }

  protected static String nodeTypeToString(int type) {
    switch (type) {
      case Node.ELEMENT_NODE:
        return "ELEMENT_NODE";
      case Node.ATTRIBUTE_NODE:
        return "ATTRIBUTE_NODE";
      case Node.TEXT_NODE:
        return "TEXT_NODE";
      case Node.CDATA_SECTION_NODE:
        return "CDATA_SECTION_NODE";
      case Node.ENTITY_REFERENCE_NODE:
        return "ENTITY_REFERENCE_NODE";
      case Node.ENTITY_NODE:
        return "ENTITY_NODE";
      case Node.PROCESSING_INSTRUCTION_NODE:
        return "PROCESSING_INSTRUCTION_NODE";
      case Node.COMMENT_NODE:
        return "COMMENT_NODE";
      case Node.DOCUMENT_NODE:
        return "DOCUMENT_NODE";
      case Node.DOCUMENT_TYPE_NODE:
        return "DOCUMENT_TYPE_NODE";
      case Node.DOCUMENT_FRAGMENT_NODE:
        return "DOCUMENT_FRAGMENT_NODE";
      case Node.NOTATION_NODE:
        return "NOTATION_NODE";
    }
    return "UNKNOWN NODE TYPE";
  }

  public static void compareNodes(Node origin, Node target, boolean clone, boolean deep) {
    //System.out.println( "Comparing nodes " + origin + ", " + target );
    if (!(origin == null && target == null)) {
      if (origin != null) {
        Assert.assertNotNull("NodeMatcher : Missing node, origin tagname : " + origin.getNodeName(), target);
      } else {
        Assert.assertNull("NodeMatcher : extra node: " + (target != null ? target.getNodeName() : ""), target);
      }
      // Node Type
      Assert.assertEquals("NodeMatcher : node types do not match", nodeTypeToString(origin.getNodeType()),
              nodeTypeToString(target.getNodeType()));
      // Node name
      Assert.assertEquals("NodeMatcher : node names do not match.", origin.getNodeName(), target.getNodeName());
      // Node value
      Assert.assertEquals("NodeMatcher : node values do not match.", origin.getNodeValue(), target.getNodeValue());
      // TODO : Find a way to compare readonly
      // Local name
      Assert.assertEquals("NodeMatcher : local names do not match.", origin.getLocalName(), target.getLocalName());
      // Namespace URI
      Assert.assertEquals("NodeMatcher : namespace URI's do not match (localName=" + origin.getLocalName() + ")."
              , origin.getNamespaceURI(), target.getNamespaceURI());
      // Prefix
      Assert.assertEquals("NodeMatcher : prefixes do not match.", origin.getPrefix(), target.getPrefix());
      // hasAttributes
      if (origin.hasAttributes()) {
        Assert.assertTrue("NodeMatcher : hasAttributes() should return true for node with name " + origin.getNodeName()
                , target.hasAttributes());
      }
      // hasChildren
      if (clone || (origin.getNodeType() != Node.ENTITY_REFERENCE_NODE)) {
        if (origin.hasChildNodes() && deep) {
          Assert.assertTrue("NodeMatcher : hasChildren() should return true for node with name " + origin.getNodeName()
                 , target.hasChildNodes());
        }
      }
      // Attributes
      if (origin.getNodeType() == Node.ELEMENT_NODE) {
        //System.out.println("Comparing elments");
        compareNamedNodeMaps(origin.getAttributes(), target.getAttributes(), clone, deep);
      }
      if (origin.getNodeType() == Node.ATTRIBUTE_NODE) {
        //System.out.println("Comparing attributes");
        compareAttrs((Attr) origin, (Attr) target, clone, deep);
      }
      if (origin.getNodeType() == Node.ENTITY_NODE) {
        //System.out.println("Comparing entity");
        compareEntities((Entity) origin, (Entity) target);
      }
      if (origin.getNodeType() == Node.PROCESSING_INSTRUCTION_NODE) {
        //System.out.println("Comparing pi");
        comparePIs((ProcessingInstruction) origin, (ProcessingInstruction) target);
      }
      if (origin.getNodeType() == Node.NOTATION_NODE) {
        //System.out.println("Comparing notation");
        compareNotations((Notation) origin, (Notation) target);
      }
      if (origin.getNodeType() == Node.DOCUMENT_NODE) {
        //System.out.println("Comparing documents");
        compareDocuments((Document) origin, (Document) target, clone, deep);
      }
      // TODO (ADQ) : Add special case for libraries (comparing catalogs)
      if ((deep) && (clone || (origin.getNodeType() != Node.ENTITY_REFERENCE_NODE))) {
        //System.out.println("Comparing children of "+origin.getNodeName());
        // Compare children
        Node originPointer = origin.getFirstChild();
        Node targetPointer = target.getFirstChild();
        while (originPointer != null) {
          //System.out.println("Comparing child "+originPointer.getNodeName());
          Assert.assertNotNull("NodeMatcher : Child list for node does not match, missing node "
                  + originPointer.getNodeName(), targetPointer);
          compareNodes(originPointer, targetPointer, clone, deep);
          originPointer = originPointer.getNextSibling();
          targetPointer = targetPointer.getNextSibling();
        }
        if (targetPointer != null && originPointer == null) {
          Assert.fail("NodeMatcher : Child list does not match, node should not be there :"
                  + targetPointer.getNodeName() + ", " + targetPointer.getNodeValue());
        }
      }
    }
  }

  public static void compareNodeLists(NodeList origin, NodeList target, boolean clone, boolean deep) {
    if (origin != null) {
      Assert.assertNotNull("NodeMatcher : expected nodelist not found", target);
      Assert.assertEquals("NodeMatcher : nodelist item count does not match.", origin.getLength(), target.getLength());

      for (int i = 0; i < origin.getLength(); i++) {
        Node originNode = origin.item(i);
        Node targetNode = target.item(i);
        compareNodes(originNode, targetNode, clone, deep);
      }
    } else {
      Assert.assertNull("NodeMatcher : unexpected nodelist found", target);
    }
  }

  public static void compareNamedNodeMaps(NamedNodeMap origin, NamedNodeMap target, boolean clone, boolean deep) {
    if (origin != null) {
      Assert.assertNotNull("NodeMatcher : expected namednodemap not found", target);
      Assert.assertEquals("NodeMatcher : nodemap item count does not match.", origin.getLength(), target.getLength());

      for (int i = 0; i < origin.getLength(); i++) {
        Node originNode = origin.item(i);
        // Lengths checked before, so no need to check whether all targets are in origin
        Node targetNode = lookupTargetNode(target, originNode);
        compareNodes(originNode, targetNode, clone, deep);
      }
    } else {
      Assert.assertNull("NodeMatcher : unexpected namednodemap found", target);
    }
  }

  /**
   * NamedNodeMaps are unordered, so lookup the item in the other list
   * called from compareNamedNodeMaps
   */
  private static Node lookupTargetNode(NamedNodeMap target, Node originNode) {
    // Namespace or not namespace?
    if (originNode.getLocalName() == null) {
      return target.getNamedItem(originNode.getNodeName());
    } else {
      return target.getNamedItemNS(originNode.getNamespaceURI(), originNode.getLocalName());
    }
  }

  protected static void compareAttrs(Attr origin, Attr target, boolean clone, boolean deep) {
    if (clone) {
      if (origin.getSpecified()) {
        Assert.assertTrue("NodeMatcher : attribute specified does not match", target.getSpecified());
      }
    }
  }

  protected static void compareEntities(Entity origin, Entity target) {
    Assert.assertEquals("NodeMatcher : entity public id's do not match.", origin.getPublicId(), target.getPublicId());
    Assert.assertEquals("NodeMatcher : entity system id's do not match.", origin.getSystemId(), target.getSystemId());
    Assert.assertEquals("NodeMatcher : entity notation names do not match.", origin.getNotationName()
            , target.getNotationName());
  }

  protected static void comparePIs(ProcessingInstruction origin, ProcessingInstruction target) {
    Assert.assertEquals("NodeMatcher : processing instruction targets do not match."
            , origin.getTarget(), target.getTarget());
    Assert.assertEquals("NodeMatcher : processing instruction data does not match."
            , origin.getData(), target.getData());
  }

  protected static void compareNotations(Notation origin, Notation target) {
    Assert.assertEquals("NodeMatcher : notation public id's do not match.", origin.getPublicId(), target.getPublicId());
    Assert.assertEquals("NodeMatcher : notation system id's do not match.", origin.getSystemId(), target.getSystemId());
  }

  /**
   * Compare two doctype nodes.
   * Presently, this matching is only used from TreeDiffTest, maybe this should be enabled in
   * compareNodes in this method as well (but this may lead to problems with Xerces documents, which probably
   * have slightly different information).
   */
  public static void compareDocTypes(DocumentType origin, DocumentType target, boolean clone, boolean deep) {
    if (origin == null) {
      Assert.assertNull("NodeMatcher: One doctype is null and one is not", target);
    } else {
      Assert.assertNotNull("NodeMatcher: One doctype is null and one is not", target);
    }
    Assert.assertEquals("NodeMatcher : document type public id's do not match."
            , origin.getPublicId(), target.getPublicId());
    Assert.assertEquals("NodeMatcher : document type system id's do not match."
            , origin.getSystemId(), target.getSystemId());
    Assert.assertEquals("NodeMatcher : document type names do not match."
            , origin.getName(), target.getName());
    compareNamedNodeMaps(origin.getEntities(), target.getEntities(), clone, deep);
    compareNamedNodeMaps(origin.getNotations(), target.getNotations(), clone, deep);
  }

  protected static void compareDocuments(Document origin, Document target, boolean clone, boolean deep) {
    DocumentType originDocumentType = origin.getDoctype();
    DocumentType targetDocumentType = target.getDoctype();
    compareNodes(originDocumentType, targetDocumentType, clone, deep);
  }

}
